@@ -27,6 +27,7 @@ def stream_unzip(zipfile_chunks, chunk_size=65536):
                 yield chunk[offset:offset + to_yield]
                 amt -= to_yield
                 offset += to_yield % len(chunk)
+                chunk = chunk if offset else b''
 
             # Yield the rest as it comes in
             while amt:
@@ -56,6 +57,7 @@ def stream_unzip(zipfile_chunks, chunk_size=65536):
             chunk = b''
             if prev_chunk:
                 yield prev_chunk[prev_offset:]
+            prev_chunk = b''
 
             while True:
                 try:
@@ -100,6 +102,9 @@ def stream_unzip(zipfile_chunks, chunk_size=65536):
         if compressed_size == zip64_compressed_size:
             uncompressed_size, compressed_size = Struct('<QQ').unpack(extra[zip64_size_signature])
 
+        if flags == b'\x08\x00':
+            uncompressed_size = None
+
         def _decompress_deflate():
             dobj = zlib.decompressobj(wbits=-zlib.MAX_WBITS)
 
@@ -116,8 +121,10 @@ def stream_unzip(zipfile_chunks, chunk_size=65536):
                 if dobj.eof:
                     break
 
-            if dobj.unused_data:
-                return_unused(dobj.unused_data)
+            if not dobj.eof:
+                raise ValueError('Fewer bytes than expected in zip')
+
+            return_unused(dobj.unused_data)
 
             # Read the data descriptor
             if flags == b'\x08\x00':
@@ -131,9 +138,6 @@ def stream_unzip(zipfile_chunks, chunk_size=65536):
             read_multiple_chunks(compressed_size) if compression == 0 else \
             _decompress_deflate()
 
-        if uncompressed_size == 0:
-            uncompressed_size = None
-
         return file_name, uncompressed_size, uncompressed_bytes
 
     while True:
